//

// Created by hruskraj on 28. 4. 2020
//

#include <catch2/catch.hpp>

#include <alib/set>
#include <string/LinearString.h>
#include <stringology/cover/ApproximateEnhancedCoversComputation.h>

TEST_CASE ( "Approximate Enhanced Covers", "[unit][stringology][cover]" ) {
	SECTION ( "One simple enhanced cover" ) {
		string::LinearString < char > pattern ( "abacaccababa" );
		unsigned int k = 1;

		ext::set < string::LinearString < char > > refSet;
		string::LinearString < char > a1 ( "aba" );

		refSet.insert ( a1 );

		ext::set < string::LinearString < char > > result = stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k );

		CHECK ( result == refSet );
	}

	SECTION ( "Multiple enhanced covers" ) {
		string::LinearString < char > pattern ( "ababaccababa" );
		unsigned int k = 2;

		ext::set < string::LinearString < char > > refSet;
		string::LinearString < char > a1 ( "aba" );
		string::LinearString < char > a2 ( "ababa" );

		refSet.insert ( a1 );
		refSet.insert ( a2 );

		ext::set < string::LinearString < char > > result = stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k );

		CHECK ( result == refSet );
	}

	SECTION ( "Empty string" ) {
		string::LinearString < char > pattern ( "" );
		unsigned int k = 0;

		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = 1;
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = 15;
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = -1;
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );
	}

	SECTION ( "k = 0 (enhanced covers)" ) {
		unsigned int k = 0;

		string::LinearString < char > pattern ( "x" );

		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		string::LinearString < char > pattern2 ( "ab" );
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern2, k ).empty ( ) );

		string::LinearString < char > pattern3 ( "aa" );
		string::LinearString < char > a1 ( "a" );
		ext::set < string::LinearString < char > > refSet;
		refSet.insert ( a1 );
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern3, k ) == refSet );

		string::LinearString < char > pattern4 ( "aba" );
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern4, k ) == refSet );

		string::LinearString < char > pattern5 ( "aaa" );
		string::LinearString < char > a2 ( "aa" );
		refSet.insert ( a2 );
		CHECK ( stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern5, k ) == refSet );
	}

	SECTION ( "Different k" ) {
		string::LinearString < char > pattern ( "adesgvsade" );
		unsigned int k = 3;

		ext::set < string::LinearString < char > > result = stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k );

		CHECK ( result.empty ( ) );

		k = 2;
		ext::set < string::LinearString < char > > refSet;
		string::LinearString < char > a1 ( "ade" );
		refSet.insert ( a1 );

		result = stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern, k );
		CHECK ( result == refSet );

		string::LinearString < char > pattern2 ( "abcd" );
		k = 1;

		result = stringology::cover::ApproximateEnhancedCoversComputation::compute ( pattern2, k );
		CHECK ( result.empty ( ) );
	}
}
