//

// Created by hruskraj on 4. 5. 2020
//

#include <catch2/catch.hpp>

#include <alib/set>
#include <string/LinearString.h>
#include <stringology/cover/RelaxedApproximateEnhancedCoversComputation.h>

TEST_CASE ( "Relaxed Approximate Enhanced Covers", "[unit][stringology][cover]" ) {
	SECTION ( "One simple enhanced cover" ) {
		string::LinearString < char > pattern ( "abcabc" );
		unsigned int k = 1;

		ext::set < string::LinearString < char > > refSet;
		string::LinearString < char > a1 ( "abc" );

		refSet.insert ( a1 );

		ext::set < string::LinearString < char > > result = stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k );

		CHECK ( result == refSet );
	}

	SECTION ( "Multiple relaxed approximate enhanced covers" ) {
		string::LinearString < char > pattern ( "abacaccababa" );
		unsigned int k = 2;

		ext::set < string::LinearString < char > > refSet;
		string::LinearString < char > a1 ( "cca" );
		string::LinearString < char > a2 ( "aba" );
		string::LinearString < char > a3 ( "aca" );
		string::LinearString < char > a4 ( "acc" );
		string::LinearString < char > a5 ( "abaca" );
		string::LinearString < char > a6 ( "ababa" );

		refSet.insert ( a1 );
		refSet.insert ( a2 );
		refSet.insert ( a3 );
		refSet.insert ( a4 );
		refSet.insert ( a5 );
		refSet.insert ( a6 );

		ext::set < string::LinearString < char > > result = stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k );

		CHECK ( result == refSet );
	}

	SECTION ( "Empty string" ) {
		string::LinearString < char > pattern ( "" );
		unsigned int k = 0;

		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = 1;
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = 15;
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		k = -1;
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );
	}

	SECTION ( "k = 0 (enhanced covers)" ) {
		unsigned int k = 0;

		string::LinearString < char > pattern ( "x" );

		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k ).empty ( ) );

		string::LinearString < char > pattern2 ( "ab" );
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern2, k ).empty ( ) );

		string::LinearString < char > pattern3 ( "aa" );
		string::LinearString < char > a1 ( "a" );
		ext::set < string::LinearString < char > > refSet;
		refSet.insert ( a1 );
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern3, k ) == refSet );

		string::LinearString < char > pattern4 ( "aba" );
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern4, k ) == refSet );

		string::LinearString < char > pattern5 ( "aaa" );
		string::LinearString < char > a2 ( "aa" );
		refSet.insert ( a2 );
		CHECK ( stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern5, k ) == refSet );
	}

	SECTION ( "Different k" ) {
		string::LinearString < char > pattern ( "abbbabab" );
		unsigned int k = 4;

		string::LinearString < char > a1 ( "bbaba" );
		string::LinearString < char > a2 ( "bbbaba" );
		string::LinearString < char > a3 ( "babab" );
		string::LinearString < char > a4 ( "bbbab" );
		string::LinearString < char > a5 ( "abbbab" );
		string::LinearString < char > a6 ( "abbba" );
		string::LinearString < char > a7 ( "bbabab" );

		ext::set < string::LinearString < char > > refSet1;

		refSet1.insert ( a1 );
		refSet1.insert ( a2 );
		refSet1.insert ( a3 );
		refSet1.insert ( a4 );
		refSet1.insert ( a5 );
		refSet1.insert ( a6 );
		refSet1.insert ( a7 );

		ext::set < string::LinearString < char > > result = stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k );
		CHECK ( result == refSet1 );

		k = 3;
		string::LinearString < char > b1 ( "abbb" );
		string::LinearString < char > b2 ( "bbba" );
		string::LinearString < char > b3 ( "bbab" );
		string::LinearString < char > b4 ( "bbbab" );
		string::LinearString < char > b5 ( "abab" );
		string::LinearString < char > b6 ( "bbabab" );
		string::LinearString < char > b7 ( "abbbab" );

		ext::set < string::LinearString < char > > refSet2;
		refSet2.insert ( b1 );
		refSet2.insert ( b2 );
		refSet2.insert ( b3 );
		refSet2.insert ( b4 );
		refSet2.insert ( b5 );
		refSet2.insert ( b6 );
		refSet2.insert ( b7 );

		result = stringology::cover::RelaxedApproximateEnhancedCoversComputation::compute ( pattern, k );
		CHECK ( result == refSet2 );
	}
}
